@@ -3,9 +3,10 @@
 /***********************************************************************
 *  This code is part of GLPK (GNU Linear Programming Kit).
 *
-*  Copyright (C) 2000, 01, 02, 03, 04, 05, 06, 07 Andrew Makhorin,
-*  Department for Applied Informatics, Moscow Aviation Institute,
-*  Moscow, Russia. All rights reserved. E-mail: <mao@mai2.rcnet.ru>.
+*  Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
+*  2009, 2010 Andrew Makhorin, Department for Applied Informatics,
+*  Moscow Aviation Institute, Moscow, Russia. All rights reserved.
+*  E-mail: <mao@gnu.org>.
 *
 *  GLPK is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
@@ -23,7 +24,7 @@
 
 #include "glplib.h"
 #include "glpssx.h"
-#define fault xfault1
+#define xfault xerror
 
 /*----------------------------------------------------------------------
 // ssx_create - create simplex solver workspace.
@@ -41,12 +42,12 @@ SSX *ssx_create(int m, int n, int nnz)
 {     SSX *ssx;
       int i, j, k;
       if (m < 1)
-         fault("ssx_create: m = %d; invalid number of rows", m);
+         xfault("ssx_create: m = %d; invalid number of rows\n", m);
       if (n < 1)
-         fault("ssx_create: n = %d; invalid number of columns", n);
+         xfault("ssx_create: n = %d; invalid number of columns\n", n);
       if (nnz < 0)
-         fault("ssx_create: nnz = %d; invalid number of non-zero constr"
-            "aint coefficients", nnz);
+         xfault("ssx_create: nnz = %d; invalid number of non-zero const"
+            "raint coefficients\n", nnz);
       ssx = xmalloc(sizeof(SSX));
       ssx->m = m;
       ssx->n = n;
